@@ -9,7 +9,6 @@ import (
 	"io/ioutil"
 	"os"
 	"os/exec"
-	"path"
 	"path/filepath"
 	"reflect"
 	"strconv"
@@ -27,7 +26,6 @@ import (
 	"github.com/opencontainers/runc/libcontainer/cgroups/devices"
 	"github.com/opencontainers/runc/libcontainer/cgroups/fscommon"
 	"github.com/opencontainers/runc/libcontainer/configs"
-	"github.com/prometheus/procfs"
 
 	"k8s.io/apimachinery/pkg/types"
 
@@ -37,7 +35,7 @@ import (
 
 var (
 	deviceBasePath = func(podUID types.UID) string {
-		return fmt.Sprintf("/proc/1/root/var/lib/kubelet/pods/%s/volumeDevices", string(podUID))
+		return fmt.Sprintf("/proc/1/root/var/lib/kubelet/pods/%s/volumes/kubernetes.io~empty-dir/hotplug-disks", string(podUID))
 	}
 
 	sourcePodBasePath = func(podUID types.UID) string {
@@ -79,10 +77,6 @@ var (
 	isolationDetector = func(path string) isolation.PodIsolationDetector {
 		return isolation.NewSocketBasedIsolationDetector(path, cgroup.NewParser())
 	}
-
-	procMounts = func(pid int) ([]*procfs.MountInfo, error) {
-		return procfs.GetProcMounts(pid)
-	}
 )
 
 //go:generate mockgen -source $GOFILE -package=$GOPACKAGE -destination=generated_mock_$GOFILE
@@ -93,12 +87,14 @@ type volumeMounter struct {
 	mountRecords         map[types.UID]*vmiMountTargetRecord
 	mountRecordsLock     sync.Mutex
 	skipSafetyCheck      bool
+	hotplugDiskManager   hotplugdisk.HotplugDiskManagerInterface
 }
 
 // VolumeMounter is the interface used to mount and unmount volumes to/from a running virtlauncher pod.
 type VolumeMounter interface {
 	// Mount any new volumes defined in the VMI
 	Mount(vmi *v1.VirtualMachineInstance) error
+	MountFromPod(vmi *v1.VirtualMachineInstance, sourceUID types.UID) error
 	// Unmount any volumes no longer defined in the VMI
 	Unmount(vmi *v1.VirtualMachineInstance) error
 	//UnmountAll cleans up all hotplug volumes
@@ -121,6 +117,7 @@ func NewVolumeMounter(isoDetector isolation.PodIsolationDetector, mountStateDir
 		podIsolationDetector: isoDetector,
 		mountRecords:         make(map[types.UID]*vmiMountTargetRecord),
 		mountStateDir:        mountStateDir,
+		hotplugDiskManager:   hotplugdisk.NewHotplugDiskManager(),
 	}
 }
 
@@ -239,8 +236,26 @@ func (m *volumeMounter) writePathToMountRecord(path string, vmi *v1.VirtualMachi
 	return nil
 }
 
-func (m *volumeMounter) Mount(vmi *v1.VirtualMachineInstance) error {
+func (m *volumeMounter) mountHotplugVolume(vmi *v1.VirtualMachineInstance, volumeName string, sourceUID types.UID, record *vmiMountTargetRecord) error {
 	logger := log.DefaultLogger()
+	logger.V(4).Infof("Hotplug check volume name: %s", volumeName)
+	if sourceUID != types.UID("") {
+		if m.isBlockVolume(sourceUID, volumeName) {
+			logger.V(4).Infof("Mounting block volume: %s", volumeName)
+			if err := m.mountBlockHotplugVolume(vmi, volumeName, sourceUID, record); err != nil {
+				return err
+			}
+		} else {
+			logger.V(4).Infof("Mounting file system volume: %s", volumeName)
+			if err := m.mountFileSystemHotplugVolume(vmi, volumeName, sourceUID, record); err != nil {
+				return err
+			}
+		}
+	}
+	return nil
+}
+
+func (m *volumeMounter) Mount(vmi *v1.VirtualMachineInstance) error {
 	record, err := m.getMountTargetRecord(vmi)
 	if err != nil {
 		return err
@@ -250,35 +265,39 @@ func (m *volumeMounter) Mount(vmi *v1.VirtualMachineInstance) error {
 			// Skip non hotplug volumes
 			continue
 		}
-		logger.V(4).Infof("Hotplug check volume name: %s", volumeStatus.Name)
 		sourceUID := volumeStatus.HotplugVolume.AttachPodUID
-		if sourceUID != types.UID("") {
-			if m.isBlockVolume(sourceUID) {
-				logger.V(4).Infof("Mounting block volume: %s", volumeStatus.Name)
-				if err := m.mountBlockHotplugVolume(vmi, volumeStatus.Name, sourceUID, record); err != nil {
-					return err
-				}
-			} else {
-				logger.V(4).Infof("Mounting file system volume: %s", volumeStatus.Name)
-				if err := m.mountFileSystemHotplugVolume(vmi, volumeStatus.Name, sourceUID, record); err != nil {
-					return err
-				}
-			}
+		if err := m.mountHotplugVolume(vmi, volumeStatus.Name, sourceUID, record); err != nil {
+			return err
 		}
 	}
+	return nil
+}
 
+func (m *volumeMounter) MountFromPod(vmi *v1.VirtualMachineInstance, sourceUID types.UID) error {
+	record, err := m.getMountTargetRecord(vmi)
+	if err != nil {
+		return err
+	}
+	for _, volumeStatus := range vmi.Status.VolumeStatus {
+		if volumeStatus.HotplugVolume == nil {
+			// Skip non hotplug volumes
+			continue
+		}
+		if err := m.mountHotplugVolume(vmi, volumeStatus.Name, sourceUID, record); err != nil {
+			return err
+		}
+	}
 	return nil
 }
 
 // isBlockVolume checks if the volumeDevices directory exists in the pod path, we assume there is a single volume associated with
 // each pod, we use this knowledge to determine if we have a block volume or not.
-func (m *volumeMounter) isBlockVolume(sourceUID types.UID) bool {
+func (m *volumeMounter) isBlockVolume(sourceUID types.UID, volumeName string) bool {
 	// Check if the volumeDevices directory exists in the attachment pod, if so, its a block device, otherwise its file system.
 	if sourceUID != types.UID("") {
-		devicePath := deviceBasePath(sourceUID)
+		devicePath := filepath.Join(deviceBasePath(sourceUID), volumeName)
 		info, err := os.Stat(devicePath)
 		if err != nil {
-			log.Log.V(4).Infof("%s pod does not contain a block device %v", sourceUID, err)
 			return false
 		}
 		return info.IsDir()
@@ -292,19 +311,21 @@ func (m *volumeMounter) mountBlockHotplugVolume(vmi *v1.VirtualMachineInstance,
 		// This is not the node the pod is running on.
 		return nil
 	}
-	targetPath, err := hotplugdisk.GetHotplugTargetPodPathOnHost(virtlauncherUID)
+	targetPath, err := m.hotplugDiskManager.GetHotplugTargetPodPathOnHost(virtlauncherUID)
 	if err != nil {
 		return err
 	}
 
 	deviceName := filepath.Join(targetPath, volume)
 
+	isMigrationInProgress := vmi.Status.MigrationState != nil && !vmi.Status.MigrationState.Completed
+
 	if isBlockExists, _ := isBlockDevice(deviceName); !isBlockExists {
 		computeCGroupPath, err := m.getTargetCgroupPath(vmi)
 		if err != nil {
 			return err
 		}
-		sourceMajor, sourceMinor, permissions, err := m.getSourceMajorMinor(sourceUID)
+		sourceMajor, sourceMinor, permissions, err := m.getSourceMajorMinor(sourceUID, volume)
 		if err != nil {
 			return err
 		}
@@ -318,13 +339,13 @@ func (m *volumeMounter) mountBlockHotplugVolume(vmi *v1.VirtualMachineInstance,
 		if _, err = m.createBlockDeviceFile(deviceName, sourceMajor, sourceMinor, permissions); err != nil {
 			return err
 		}
-	} else if isBlockExists && !m.volumeStatusReady(volume, vmi) {
+	} else if isBlockExists && (!m.volumeStatusReady(volume, vmi) || isMigrationInProgress) {
 		// Block device exists already, but the volume is not ready yet, ensure that the device is allowed.
 		computeCGroupPath, err := m.getTargetCgroupPath(vmi)
 		if err != nil {
 			return err
 		}
-		sourceMajor, sourceMinor, _, err := m.getSourceMajorMinor(sourceUID)
+		sourceMajor, sourceMinor, _, err := m.getSourceMajorMinor(sourceUID, volume)
 		if err != nil {
 			return err
 		}
@@ -339,7 +360,7 @@ func (m *volumeMounter) volumeStatusReady(volumeName string, vmi *v1.VirtualMach
 	for _, volumeStatus := range vmi.Status.VolumeStatus {
 		if volumeStatus.Name == volumeName && volumeStatus.HotplugVolume != nil {
 			if volumeStatus.Phase != v1.VolumeReady {
-				log.DefaultLogger().Infof("Volume %s is not ready, but the target block device exists", volumeName)
+				log.DefaultLogger().V(4).Infof("Volume %s is not ready, but the target block device exists", volumeName)
 			}
 			return volumeStatus.Phase == v1.VolumeReady
 		}
@@ -348,11 +369,11 @@ func (m *volumeMounter) volumeStatusReady(volumeName string, vmi *v1.VirtualMach
 	return true
 }
 
-func (m *volumeMounter) getSourceMajorMinor(sourceUID types.UID) (int64, int64, string, error) {
+func (m *volumeMounter) getSourceMajorMinor(sourceUID types.UID, volumeName string) (int64, int64, string, error) {
 	result := make([]int64, 2)
 	perms := ""
 	if sourceUID != types.UID("") {
-		basepath := deviceBasePath(sourceUID)
+		basepath := filepath.Join(deviceBasePath(sourceUID), volumeName)
 		err := filepath.Walk(basepath, func(filePath string, info os.FileInfo, err error) error {
 			if info != nil && !info.IsDir() {
 				// Walk doesn't follow symlinks which is good because I need to massage symlinks
@@ -535,29 +556,30 @@ func (m *volumeMounter) createBlockDeviceFile(deviceName string, major, minor in
 }
 
 func (m *volumeMounter) mountFileSystemHotplugVolume(vmi *v1.VirtualMachineInstance, volume string, sourceUID types.UID, record *vmiMountTargetRecord) error {
-	sourcePath, err := m.getSourcePodFilePath(sourceUID, vmi, volume)
-	if err != nil {
-		log.DefaultLogger().Infof("Error finding source path: %v", err)
-		return nil
-	}
-
 	virtlauncherUID := m.findVirtlauncherUID(vmi)
 	if virtlauncherUID == "" {
 		// This is not the node the pod is running on.
 		return nil
 	}
-	targetPath, err := hotplugdisk.GetFileSystemDiskTargetPathFromHostView(virtlauncherUID, volume, true)
+	targetDisk, err := m.hotplugDiskManager.GetFileSystemDiskTargetPathFromHostView(virtlauncherUID, volume, true)
 	if err != nil {
 		return err
 	}
 
-	if isMounted, err := isMounted(targetPath); err != nil {
-		return fmt.Errorf("failed to determine if %s is already mounted: %v", targetPath, err)
+	if isMounted, err := isMounted(targetDisk); err != nil {
+		return fmt.Errorf("failed to determine if %s is already mounted: %v", targetDisk, err)
 	} else if !isMounted {
-		if err := m.writePathToMountRecord(targetPath, vmi, record); err != nil {
+		sourcePath, err := m.getSourcePodFilePath(sourceUID, vmi, volume)
+		if err != nil {
+			log.DefaultLogger().V(3).Infof("Error getting source path: %v", err)
+			// We are eating the error to avoid spamming the log with errors, it might take a while for the volume
+			// to get mounted on the node, and this will error until the volume is mounted.
+			return nil
+		}
+		if err := m.writePathToMountRecord(targetDisk, vmi, record); err != nil {
 			return err
 		}
-		if out, err := mountCommand(sourcePath, targetPath); err != nil {
+		if out, err := mountCommand(filepath.Join(sourcePath, "disk.img"), targetDisk); err != nil {
 			return fmt.Errorf("failed to bindmount hotplug-disk %v: %v : %v", volume, string(out), err)
 		}
 	} else {
@@ -566,55 +588,59 @@ func (m *volumeMounter) mountFileSystemHotplugVolume(vmi *v1.VirtualMachineInsta
 	return nil
 }
 
-func (m *volumeMounter) findVirtlauncherUID(vmi *v1.VirtualMachineInstance) types.UID {
-	if len(vmi.Status.ActivePods) == 1 {
-		for k := range vmi.Status.ActivePods {
-			return k
+func (m *volumeMounter) findVirtlauncherUID(vmi *v1.VirtualMachineInstance) (uid types.UID) {
+	cnt := 0
+	for podUID := range vmi.Status.ActivePods {
+		_, err := m.hotplugDiskManager.GetHotplugTargetPodPathOnHost(podUID)
+		if err == nil {
+			uid = podUID
+			cnt++
 		}
 	}
+	if cnt == 1 {
+		return
+	}
 	// Either no pods, or multiple pods, skip.
 	return types.UID("")
 }
 
 func (m *volumeMounter) getSourcePodFilePath(sourceUID types.UID, vmi *v1.VirtualMachineInstance, volume string) (string, error) {
-	diskPath := ""
-	if sourceUID != types.UID("") {
-		basepath := sourcePodBasePath(sourceUID)
-		err := filepath.Walk(basepath, func(filePath string, info os.FileInfo, err error) error {
-			if path.Base(filePath) == "disk.img" {
-				// Found disk image
-				diskPath = path.Dir(filePath)
-				return io.EOF
-			}
-			return nil
-		})
-		if err != nil && err != io.EOF {
-			return diskPath, err
-		}
+	iso := isolationDetector("/path")
+	isoRes, err := iso.DetectForSocket(vmi, socketPath(sourceUID))
+	if err != nil {
+		return "", err
 	}
-	if diskPath == "" {
-		// Unfortunately I cannot use this approach for all storage, for instance in ceph the mount.Root is / which is obviously
-		// not the path we want to mount. So we stick with try sourcePodBasePath first, then if not found try mountinfo.
-		iso := isolationDetector("/path")
-		isoRes, err := iso.DetectForSocket(vmi, socketPath(sourceUID))
-		if err != nil {
-			return "", err
-		}
-		mounts, err := procMounts(isoRes.Pid())
-		if err != nil {
-			return "", err
-		}
-		for _, mount := range mounts {
-			if mount.MountPoint == "/pvc" {
-				return mount.Root, nil
+	findmounts, err := LookupFindmntInfoByVolume(volume, isoRes.Pid())
+	if err != nil {
+		return "", err
+	}
+	for _, findmnt := range findmounts {
+		if filepath.Base(findmnt.Target) == volume {
+			source := findmnt.GetSourcePath()
+			isBlock, _ := isBlockDevice(filepath.Join(util.HostRootMount, source))
+			if _, err := os.Stat(filepath.Join(util.HostRootMount, source)); os.IsNotExist(err) || isBlock {
+				// file not found, or block device, or directory check if we can find the mount.
+				deviceFindMnt, err := LookupFindmntInfoByDevice(source)
+				if err != nil {
+					// Try the device found from the source
+					deviceFindMnt, err = LookupFindmntInfoByDevice(findmnt.GetSourceDevice())
+					if err != nil {
+						return "", err
+					}
+					// Check if the path was relative to the device.
+					if _, err := os.Stat(filepath.Join(util.HostRootMount, source)); err != nil {
+						return filepath.Join(deviceFindMnt[0].Target, source), nil
+					}
+					return "", err
+				}
+				return deviceFindMnt[0].Target, nil
+			} else {
+				return source, nil
 			}
 		}
 	}
-	if diskPath == "" {
-		// Did not find the disk image file, return error
-		return diskPath, fmt.Errorf("Unable to find source disk image path for pod %s", sourceUID)
-	}
-	return diskPath, nil
+	// Did not find the disk image file, return error
+	return "", fmt.Errorf("unable to find source disk image path for pod %s", sourceUID)
 }
 
 // Unmount unmounts all hotplug disk that are no longer part of the VMI
@@ -634,7 +660,7 @@ func (m *volumeMounter) Unmount(vmi *v1.VirtualMachineInstance) error {
 		currentHotplugPaths := make(map[string]types.UID, 0)
 		virtlauncherUID := m.findVirtlauncherUID(vmi)
 
-		basePath, err := hotplugdisk.GetHotplugTargetPodPathOnHost(virtlauncherUID)
+		basePath, err := m.hotplugDiskManager.GetHotplugTargetPodPathOnHost(virtlauncherUID)
 		if err != nil {
 			return err
 		}
@@ -642,11 +668,11 @@ func (m *volumeMounter) Unmount(vmi *v1.VirtualMachineInstance) error {
 			if volumeStatus.HotplugVolume == nil {
 				continue
 			}
-			if m.isBlockVolume(volumeStatus.HotplugVolume.AttachPodUID) {
+			if m.isBlockVolume(volumeStatus.HotplugVolume.AttachPodUID, volumeStatus.Name) {
 				path := filepath.Join(basePath, volumeStatus.Name)
 				currentHotplugPaths[path] = virtlauncherUID
 			} else {
-				path, err := hotplugdisk.GetFileSystemDiskTargetPathFromHostView(virtlauncherUID, volumeStatus.Name, false)
+				path, err := m.hotplugDiskManager.GetFileSystemDiskTargetPathFromHostView(virtlauncherUID, volumeStatus.Name, false)
 				if err != nil {
 					return err
 				}
@@ -742,12 +768,12 @@ func (m *volumeMounter) UnmountAll(vmi *v1.VirtualMachineInstance) error {
 			diskPath := entry.TargetFile
 			if m.isBlockFile(diskPath) {
 				if err := m.unmountBlockHotplugVolumes(diskPath, vmi); err != nil {
-					logger.Infof("Unable to remove block device at path %s", diskPath)
+					logger.Infof("Unable to remove block device at path %s: %v", diskPath, err)
 					// Don't return error, try next.
 				}
 			} else {
 				if err := m.unmountFileSystemHotplugVolumes(diskPath); err != nil {
-					logger.Infof("Unable to unmount volume at path %s", diskPath)
+					logger.Infof("Unable to unmount volume at path %s: %v", diskPath, err)
 					// Don't return error, try next.
 				}
 			}
@@ -766,14 +792,14 @@ func (m *volumeMounter) IsMounted(vmi *v1.VirtualMachineInstance, volume string,
 		// This is not the node the pod is running on.
 		return false, fmt.Errorf("Unable to determine virt-launcher UID")
 	}
-	targetPath, err := hotplugdisk.GetHotplugTargetPodPathOnHost(virtlauncherUID)
+	targetPath, err := m.hotplugDiskManager.GetHotplugTargetPodPathOnHost(virtlauncherUID)
 	if err != nil {
 		return false, err
 	}
-	if m.isBlockVolume(sourceUID) {
+	if m.isBlockVolume(sourceUID, volume) {
 		deviceName := filepath.Join(targetPath, volume)
 		isBlockExists, _ := isBlockDevice(deviceName)
 		return isBlockExists, nil
 	}
-	return isMounted(filepath.Join(targetPath, volume))
+	return isMounted(filepath.Join(targetPath, fmt.Sprintf("%s.img", volume)))
 }
